==> fxt-2004.10.25.tbz: include/fxt/bits/bitcopy.h <==
#if !defined  HAVE_BITCOPY_H__
#define       HAVE_BITCOPY_H__

#include "fxttypes.h"
//#include "bits/bitsperlong.h"


//: result undefined for indices i >= BITS_PER_LONG

inline ulong test_bit(ulong a, ulong i)
// Return whether bit[i] is set
{
    ulong b = 1UL << i;
    return  (a & b);
}
// -------------------------

inline ulong set_bit(ulong a, ulong i)
// Return a with bit[i] set
{
    return  (a | (1UL << i));
}
// -------------------------

inline ulong delete_bit(ulong a, ulong i)
// Return a with bit[i] set to zero
{
    return  (a & ~(1UL << i));
}
// -------------------------

inline ulong change_bit(ulong a, ulong i)
// Return a with bit[i] changed
{
    return  (a ^ (1UL << i));
}
// -------------------------


inline ulong copy_bit(ulong a, ulong isrc, ulong idst)
// Copy bit at [isrc] to position [idst]
{
    ulong v = a & (1UL << isrc);
    ulong b = 1UL << idst;
    if ( 0==v )  a &= ~b;
    else         a |=  b;
    return  a;
}
// -------------------------


inline ulong mask_copy_bit(ulong a, ulong msrc, ulong mdst)
// Copy bit according to src-mask (msrc)
//  to the bit according to the dest-mask (mdst).
// Both msrc and mdst must have exactly one bit set.
{
    ulong v = a & msrc;
    ulong b = mdst;
    if ( 0==v )  a &= ~b;
    else         a |=  b;
    return  a;
}
// -------------------------

inline ulong mask_or_bit(ulong a, ulong msrc, ulong mdst)
// Or bit according to src-mask (msrc)
//  to the bit according to the dest-mask (mdst).
// Both msrc and mdst must have exactly one bit set.
// If the mdst-position is known to be zero this function
//  is equivalent to mask_copy_bit().
{
    if ( a & msrc )  a |= mdst;
    return  a;
}
// -------------------------


#endif  // !defined HAVE_BITCOPY_H__

==> fxt-2008.08.17.tbz: include/fxt/bits/bitcopy.h <==
#if !defined  HAVE_BITCOPY_H__
#define       HAVE_BITCOPY_H__

#include "fxttypes.h"



inline ulong copy_bit(ulong a, ulong isrc, ulong idst)
// Copy bit at [isrc] to position [idst].
// Return the modified word.
{
#if 1  // Branch free:
    ulong x = ((a>>isrc) ^ (a>>idst)) & 1;  // one if bits differ
    a ^= (x<<idst); // change if bits differ
#else
    ulong v = a & (1UL << isrc);
    ulong b = 1UL << idst;
    if ( 0==v )  a &= ~b;
    else         a |=  b;
#endif
    return  a;
}
// -------------------------


inline ulong mask_copy_bit(ulong a, ulong msrc, ulong mdst)
// Copy bit according at src-mask (msrc)
//  to the bit according to the dest-mask (mdst).
// Both msrc and mdst must have exactly one bit set.
// Return the modified word.
{
#if 1  // Branch free:
    ulong x = mdst;
    if ( msrc & a )  x = 0;  // zero if source bit set (compiled to cmov)
    x ^= mdst;  // ==mdst if source bit set, else zero
    a &= ~mdst;  // clear dest bit
    a |= x;
#else
    ulong v = a & msrc;
    ulong b = mdst;
    if ( 0==v )  a &= ~b;
    else         a |=  b;
#endif
    return  a;
}
// -------------------------

inline ulong mask_or_bit(ulong a, ulong msrc, ulong mdst)
// Or bit according to src-mask (msrc)
//  to the bit according to the dest-mask (mdst).
// Both msrc and mdst must have exactly one bit set.
// If the mdst-position is known to be zero this function
//  is equivalent to mask_copy_bit().
// Return the modified word.
{
#if 1  // Branch free:
    if ( msrc & a )  mdst = 0;  // zero if source bit set (compiled to cmov)
    a |= mdst;
#else
    if ( a & msrc )  a |= mdst;
#endif
    return  a;
}
// -------------------------


#endif  // !defined HAVE_BITCOPY_H__
